import math



def bisect( f, c, Lb, Ub ):
    s = 1.0 if f.v(Ub) > f.v(Lb) else -1.0
    while ((abs(Ub - Lb) > (1.0e-15) * (abs(Ub) + abs(Lb)))
        and
          (abs(Ub) + abs(Lb) > 1.0e-14)):
              m = 0.5 * (Ub + Lb)
              if (s * (f.v(m) - c)) > 0.0:
                  Ub = m
              else:
                  Lb = m

    return 0.5 * (Ub + Lb)



def Newton( f, df, c, Lb, Ub ):
    x = Lb + 0.3 * (Ub - Lb)
    x0 = Lb + 0.6 * (Ub - Lb)
    niter = 0
    while ((abs(x - x0) > (1.0e-14) * (abs(x) + abs(x0)))
           and
           ((abs(x) + abs(x0)) > 1.0e-14)):
        x0 = x
        x = x0 + (c - f.v(x0)) / df.v(x0)
        if x < Lb:
            x = Lb
        if x > Ub:
            x = Ub
        niter += 1
        if niter > 20:      #       Newton method failed.
            break

    if niter <= 20:         #       When Newton method succeeds.
        return x    

    root = bisect( f, c, Lb, Ub )   #   When Newton method failed.
    return root

